<?php

declare(strict_types=1);

namespace Drupal\node_swapper\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\path_alias\Entity\PathAlias;
use Drupal\redirect\Entity\Redirect;

/**
 * @todo Add a description for the form.
 */
final class NodeSwapperConfirmForm extends ConfirmFormBase {

  /**
   * ID of the item to swap from.
   *
   * @var int
   */
  protected $src;

  /**
   * ID of the item to swap to.
   *
   * @var int
   */
  protected $dest;



  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'node_swapper_node_swapper_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    return $this->t('Are you sure you want to do this?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return new Url('node_swapper.node_swapper');
  }


  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $src = NULL, $dest = NULL) {
    $src_node = Node::load($src);
    $dest_node = Node::load($dest);
    $form['notice'] = [
      '#type' => 'markup',
      '#markup' => "
      <p>This will move the URL of \"{$src_node->title->value}\" to \"{$dest_node->title->value}\" and point <b>ALL</b> aliases and redirects to the new node</p>
      <p>In addition, <b>ALL</b> aliases of \"New Node\" will be deleted. Redirects will not be deleted.</p>
      ",
    ];

    $form['transaction_summary'] = [
      '#prefix' => '<strong>Transaction Summary</strong>',
      '#type' => 'table',
      '#header' => [
        ['data' => ''],
        ['data' => $this->t('Old Node')],
        ['data' => $this->t('New Node')],
      ],
      '#rows' => [
        ['data' => [
          ['data' => $this->t('Name')],
          ['data' => $src_node->title->value],
          ['data' => $dest_node->title->value]
        ]],
        ['data' => [
          ['data' => $this->t('Current URL')],
          ['data' => \Drupal::service('path_alias.manager')->getAliasByPath('/node/'.$src)],
          ['data' => \Drupal::service('path_alias.manager')->getAliasByPath('/node/'.$dest)]
        ]],
        ['data' => [
          ['data' => $this->t('New URL')],
          ['data' => \Drupal::service('path_alias.manager')->getAliasByPath('/node/'.$src) . '--old'],
          ['data' => \Drupal::service('path_alias.manager')->getAliasByPath('/node/'.$src)]
        ]],
        ['data' => [
          ['data' => $this->t('Aliases')],
          ['data' => $this->t('Will be assigned to the new node.')],
          ['data' => $this->t('All existing aliases will be replaced with those from the old node.')]
        ]],
        ['data' => [
          ['data' => $this->t('Redirects')],
          ['data' => $this->t('Will be assigned to the new node.')],
          ['data' => $this->t('Existing redirects will be unchanged. Will inherit all existing aliases from the old node.')]
        ]],
      ],
    ];

    $form['src'] = [
      '#type' => 'hidden',
      '#value' => $src_node->id(),
    ];

    $form['dest'] = [
      '#type' => 'hidden',
      '#value' => $dest_node->id(),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $src = $form_state->cleanValues()->getValue('src');
    $dest = $form_state->cleanValues()->getValue('dest');

    $src_aliases = \drupal::database()->query('
      select id
      from {path_alias}
      where path = :source
    ', array(':source' => '/node/' . $src))->fetchall();

    $src_redirects= \drupal::database()->query('
      select rid
      from {redirect}
      where redirect_redirect__uri LIKE :source
    ', array(':source' => '%node/' . $src . '%'))->fetchAll();

    $dest_aliases = \Drupal::database()->query('
      SELECT id
      FROM {path_alias}
      WHERE path = :source
    ', array(':source' => '/node/' . $dest))->fetchAll();

    // Grab the latest source alias before we change anything.
    $latest_src_alias = \Drupal::service('path_alias.manager')->getAliasByPath("/node/$src");

    // Delete destination aliases.
    forEach($dest_aliases as $da) {
      $alias = PathAlias::load($da->id);
      if(empty($alias)){continue;}
      $alias->delete();
    }

    // Point source aliases to destination.
    forEach($src_aliases as $sa) {
      $alias = PathAlias::load($sa->id);
      if(empty($alias)){continue;}
      $alias->setPath("/node/$dest");
      $alias->save();
    }

    // Create a new alias for the old node so it includes "--old".
    if(!empty($latest_src_alias)){
      $alias_for_old_page = PathAlias::create();
      $alias_for_old_page->setPath("/node/$src");
      $alias_for_old_page->setAlias($latest_src_alias . '--old');
      $alias_for_old_page->save();
    }

    // Point all source redirects to destination node.
    forEach($src_redirects as $sr){
      $redirect = Redirect::load($sr->rid);
      if(empty($redirect)){continue;}
      $redirect->setRedirect("/node/$dest");
      $redirect->save();
    }

    $this->messenger()->addStatus($this->t('Node URLs successfully swapped'));
    $form_state->setRedirectUrl(new Url('node_swapper.node_swapper'));
  }

}
